--[[

	The MIT License (MIT)

	Copyright (c) 2022 Lars Norberg

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all
	copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
	SOFTWARE.

--]]
local Addon, ns = ...
ns = LibStub("AceAddon-3.0"):NewAddon(ns, Addon, "AceConsole-3.0", "LibMoreEvents-1.0")
ns.L = LibStub("AceLocale-3.0"):GetLocale(Addon) -- Addon localization
ns.callbacks = LibStub("CallbackHandler-1.0"):New(ns, nil, nil, false) -- Addon callback handler
ns.Hider = CreateFrame("Frame"); ns.Hider:Hide()
ns.Noop = function() end
_G[Addon] = ns

-- Default settings
local defaults = {
	char = {
		actionbars = {
			enableSecondary = false
		}
	},
	global = {
		core = {
			enableDevelopmentMode = false
		},
		minimap = {
			useServerTime = false,
			useHalfClock = true
		}
	}
}

-- Lua API
local string_lower = string.lower

-- Purge deprecated settings,
-- translate to new where applicable.
local SanitizeSettings = function(db)
	if (not db) then
		return
	end
	local numBars = db.char.actionbars.numBars
	if (numBars) then
		db.char.actionbars.numBars = nil
		if (numBars > 1) then
			db.char.actionbars.enableSecondary = true
		else
			db.char.actionbars.enableSecondary = nil
		end
	end
	return db
end

ns.SwitchUI = function(self, input)
	local ui = self:GetArgs(string_lower(input))
	if (ui == "azerite") then
		if (ns.API.IsAddOnAvailable("AzeriteUI")) then
			EnableAddOn("AzeriteUI")
			DisableAddOn("DiabolicUI2")
			ReloadUI()
		end
	end
end 

ns.UpdateSettings = function(self, event, ...)
	print(event, ...)

	-- Fire callbacks to submodules.
	ns.callbacks:Fire("Saved_Settings_Updated")
end

-- Allow other modules and addons to use this.
ns.GetSettings = function(self)
	return self.db or SanitizeSettings(LibStub("AceDB-3.0"):New(Addon.."_DB", defaults, true))
end

ns.OnInitialize = function(self)

	self.db = self:GetSettings()
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateSettings")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateSettings")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateSettings")

	-- Add a command to clear the main chat frame
	-- I mainly use this to remove clutter before taking screenshots.
	-- You could theoretically put this in a macro and clear chat then screenshot.
	self:RegisterChatCommand("clear", function() ChatFrame1:Clear() end)

	-- Mimicking AzeriteUI, and adding some seamless compatibility.
	self:RegisterChatCommand("go", "SwitchUI")
	self:RegisterChatCommand("switchto", "SwitchUI")

end
